@@ -4,8 +4,20 @@ import gast
 import sys
 
 
+if sys.version_info >= (3, 13):
+    def dump(node):
+        return gast.dump(node, show_empty=True)
+else:
+    def dump(node):
+        return gast.dump(node)
+
+
 class CompatTestCase(unittest.TestCase):
 
+    def __init__(self, *args, **kwargs):
+        unittest.TestCase.__init__(self, *args, **kwargs)
+        self.maxDiff = None
+
     if sys.version_info.major == 2:
 
         def test_FunctionDef(self):
@@ -23,8 +35,9 @@ class CompatTestCase(unittest.TestCase):
                     "Name(id='y', ctx=Load(), "
                     "annotation=None, type_comment=None"
                     ")], ctx=Load()))], decorator_list="
-                    "[], returns=None, type_comment=None)], type_ignores=[])")
-            self.assertEqual(gast.dump(tree), norm)
+                    "[], returns=None, type_comment=None, type_params=[])], "
+                    "type_ignores=[])")
+            self.assertEqual(dump(tree), norm)
 
     else:
 
@@ -38,8 +51,9 @@ class CompatTestCase(unittest.TestCase):
                     "), type_comment=None)], posonlyargs="
                     "[], vararg=None, kwonlyargs=[], kw_defaults=[], kwarg="
                     "None, defaults=[]), body=[Pass()], decorator_list=[], "
-                    "returns=None, type_comment=None)], type_ignores=[])")
-            self.assertEqual(gast.dump(tree), norm)
+                    "returns=None, type_comment=None, type_params=[])], "
+                    "type_ignores=[])")
+            self.assertEqual(dump(tree), norm)
 
         def test_KeywordOnlyArgument(self):
             code = 'def foo(*, x=1): pass'
@@ -50,8 +64,9 @@ class CompatTestCase(unittest.TestCase):
                     "(id='x', ctx=Param(), annotation=None, type_comment=None"
                     ")], kw_defaults=[Constant(value=1, kind=None)], kwarg="
                     "None, defaults=[]), body=[Pass()], decorator_list=[], "
-                    "returns=None, type_comment=None)], type_ignores=[])")
-            self.assertEqual(gast.dump(tree), norm)
+                    "returns=None, type_comment=None, type_params=[])], "
+                    "type_ignores=[])")
+            self.assertEqual(dump(tree), norm)
 
         if sys.version_info.minor >= 6:
 
@@ -67,7 +82,7 @@ class CompatTestCase(unittest.TestCase):
                         "ctx=Load(), annotation=None, type_comment=None), "
                         "conversion=-1, format_spec=None)]))], "
                         "type_ignores=[])")
-                self.assertEqual(gast.dump(tree), norm)
+                self.assertEqual(dump(tree), norm)
 
             def test_JoinedStr(self):
                 code = 'e = 1; f"e = {e}"'
@@ -82,7 +97,7 @@ class CompatTestCase(unittest.TestCase):
                         "annotation=None, type_comment=None), "
                         "conversion=-1, format_spec=None)]))], "
                         "type_ignores=[])")
-                self.assertEqual(gast.dump(tree), norm)
+                self.assertEqual(dump(tree), norm)
 
         if sys.version_info.minor >= 8:
 
@@ -94,9 +109,9 @@ class CompatTestCase(unittest.TestCase):
                         "args=[], posonlyargs=[], vararg=None, kwonlyargs=[], "
                         "kw_defaults=[], kwarg=None, defaults=[]), body=["
                         "Pass()], decorator_list=[], returns=None, "
-                        "type_comment=None)], type_ignores="
+                        "type_comment=None, type_params=[])], type_ignores="
                         "[TypeIgnore(lineno=1, tag='[excuse]')])")
-                self.assertEqual(gast.dump(tree), norm)
+                self.assertEqual(dump(tree), norm)
 
             def test_PosonlyArgs(self):
                 code = 'def foo(a, /, b): pass'
@@ -108,9 +123,139 @@ class CompatTestCase(unittest.TestCase):
                         "ctx=Param(), annotation=None, type_comment=None)], "
                         "vararg=None, kwonlyargs=[], kw_defaults=[], "
                         "kwarg=None, defaults=[]), body=[Pass()], "
-                        "decorator_list=[], returns=None, type_comment=None)"
-                        "], type_ignores=[])")
-                self.assertEqual(gast.dump(tree), norm)
+                        "decorator_list=[], returns=None, type_comment=None, "
+                        "type_params=[])], type_ignores=[])")
+                self.assertEqual(dump(tree), norm)
+
+            def test_NamedExpr(self):
+                code = '(x := 1) '
+                tree = gast.parse(code)
+                compile(gast.gast_to_ast(tree), '<test>', 'exec')
+                norm = ("Module(body=[Expr(value=NamedExpr(target=Name(id='x',"
+                        " ctx=Store(), annotation=None, type_comment=None), "
+                        "value=Constant(value=1, kind=None)))], type_ignores="
+                        "[])")
+                self.assertEqual(dump(tree), norm)
+
+            if sys.version_info.minor >= 10:
+
+                def test_MatchValue(self):
+                    code = 'match v:\n  case "hello":...'
+                    tree = gast.parse(code)
+                    compile(gast.gast_to_ast(tree), '<test>', 'exec')
+                    norm = ("Module(body=[Match(subject=Name(id='v', ctx=Load"
+                            "(), annotation=None, type_comment=None), cases="
+                            "[match_case(pattern=MatchValue(value=Constant("
+                            "value='hello', kind=None)), guard=None, body="
+                            "[Expr(value=Constant(value=Ellipsis, kind=None))]"
+                            ")])], type_ignores=[])"
+                            )
+                    self.assertEqual(dump(tree), norm)
+
+                def test_MatchSingleton(self):
+                    code = 'match v:\n  case None:...'
+                    tree = gast.parse(code)
+                    compile(gast.gast_to_ast(tree), '<test>', 'exec')
+                    norm = ("Module(body=[Match(subject=Name(id='v', ctx=Load"
+                            "(), annotation=None, type_comment=None), cases=["
+                            "match_case(pattern=MatchSingleton(value=None), "
+                            "guard=None, body=[Expr(value=Constant(value="
+                            "Ellipsis, kind=None))])])], type_ignores=[])")
+                    self.assertEqual(dump(tree), norm)
+
+                def test_MatchSequence(self):
+                    code = 'match v:\n  case a, b:...'
+                    tree = gast.parse(code)
+                    compile(gast.gast_to_ast(tree), '<test>', 'exec')
+                    norm = ("Module(body=[Match(subject=Name(id='v', ctx=Load"
+                            "(), annotation=None, type_comment=None), cases="
+                            "[match_case(pattern=MatchSequence(patterns=["
+                            "MatchAs(pattern=None, name='a'), MatchAs(pattern"
+                            "=None, name='b')]), guard=None, body=[Expr(value"
+                            "=Constant(value=Ellipsis, kind=None))])])], "
+                            "type_ignores=[])")
+                    self.assertEqual(dump(tree), norm)
+
+                def test_MatchMapping(self):
+                    code = 'match v:\n  case {1: a}:...'
+                    tree = gast.parse(code)
+                    compile(gast.gast_to_ast(tree), '<test>', 'exec')
+                    norm = ("Module(body=[Match(subject=Name(id='v', ctx=Load"
+                            "(), annotation=None, type_comment=None), cases=["
+                            "match_case(pattern=MatchMapping(keys=[Constant("
+                            "value=1, kind=None)], patterns=[MatchAs(pattern"
+                            "=None, name='a')], rest=None), guard=None, body="
+                            "[Expr(value=Constant(value=Ellipsis, kind=None))]"
+                            ")])], type_ignores=[])")
+                    self.assertEqual(dump(tree), norm)
+
+                def test_MatchClass(self):
+                    code = 'match v:\n  case Cls(attr=1):...'
+                    tree = gast.parse(code)
+                    compile(gast.gast_to_ast(tree), '<test>', 'exec')
+                    norm = ("Module(body=[Match(subject=Name(id='v', ctx=Load"
+                            "(), annotation=None, type_comment=None), cases=["
+                            "match_case(pattern=MatchClass(cls=Name(id='Cls'"
+                            ", ctx=Load(), annotation=None, type_comment=None"
+                            "), patterns=[], kwd_attrs=['attr'], kwd_patterns"
+                            "=[MatchValue(value=Constant(value=1, kind=None))"
+                            "]), guard=None, body=[Expr(value=Constant(value="
+                            "Ellipsis, kind=None))])])], type_ignores=[])")
+                    self.assertEqual(dump(tree), norm)
+
+                def test_MatchStar(self):
+                    code = 'match v:\n  case [1, *other]:...'
+                    tree = gast.parse(code)
+                    compile(gast.gast_to_ast(tree), '<test>', 'exec')
+                    norm = ("Module(body=[Match(subject=Name(id='v', ctx=Load"
+                            "(), annotation=None, type_comment=None), cases=["
+                            "match_case(pattern=MatchSequence(patterns=["
+                            "MatchValue(value=Constant(value=1, kind=None)), "
+                            "MatchStar(name='other')]), guard=None, body="
+                            "[Expr(value=Constant(value=Ellipsis, kind=None)"
+                            ")])])], type_ignores=[])")
+                    self.assertEqual(dump(tree), norm)
+
+                def test_MatchAs(self):
+                    code = 'match v:\n  case 1, other:...'
+                    tree = gast.parse(code)
+                    compile(gast.gast_to_ast(tree), '<test>', 'exec')
+                    norm = ("Module(body=[Match(subject=Name(id='v', ctx=Load"
+                            "(), annotation=None, type_comment=None), cases=["
+                            "match_case(pattern=MatchSequence(patterns=["
+                            "MatchValue(value=Constant(value=1, kind=None)), "
+                            "MatchAs(pattern=None, name='other')]), guard=None"
+                            ", body=[Expr(value=Constant(value=Ellipsis, kind"
+                            "=None))])])], type_ignores=[])")
+                    self.assertEqual(dump(tree), norm)
+
+                def test_MatchOr(self):
+                    code = 'match v:\n  case 1 | 2:...'
+                    tree = gast.parse(code)
+                    compile(gast.gast_to_ast(tree), '<test>', 'exec')
+                    norm = ("Module(body=[Match(subject=Name(id='v', ctx=Load"
+                            "(), annotation=None, type_comment=None), cases=["
+                            "match_case(pattern=MatchOr(patterns=[MatchValue("
+                            "value=Constant(value=1, kind=None)), MatchValue("
+                            "value=Constant(value=2, kind=None))]), guard="
+                            "None, body=[Expr(value=Constant(value=Ellipsis, "
+                            "kind=None))])])], type_ignores=[])")
+                    self.assertEqual(dump(tree), norm)
+
+
+                if sys.version_info.minor >= 11:
+
+                    def test_TryStar(self):
+                        code = '''
+                            try: ...
+                            except *ValueError: ...'''
+                        norm = ("Module(body=[TryStar(body=[Expr(value="
+                                "Constant(value=Ellipsis))], handlers=["
+                                "ExceptHandler(type=Name(id='ValueError', ctx"
+                                "=Load()), body=[Expr(value=Constant(value="
+                                "Ellipsis))])], orelse=[], finalbody=[])], "
+                                "type_ignores=[])")
+                        pass
 
         else:
 
@@ -120,7 +265,7 @@ class CompatTestCase(unittest.TestCase):
                 compile(gast.gast_to_ast(tree), '<test>', 'exec')
                 norm = ("Module(body=[Expr(value=Constant(value=b'0012', "
                         "kind=None))], type_ignores=[])")
-                self.assertEqual(gast.dump(tree), norm)
+                self.assertEqual(dump(tree), norm)
 
     # common
 
@@ -132,7 +277,7 @@ class CompatTestCase(unittest.TestCase):
                 "type=Name(id='e', ctx=Load(), annotation=None, "
                 "type_comment=None), name=None, body=[Pass()])]"
                 ", orelse=[Pass()], finalbody=[])], type_ignores=[])")
-        self.assertEqual(gast.dump(tree), norm)
+        self.assertEqual(dump(tree), norm)
 
     def test_TryExceptNamed(self):
         code = 'try:pass\nexcept e as f:pass\nelse:pass'
@@ -143,7 +288,7 @@ class CompatTestCase(unittest.TestCase):
                 "type_comment=None), name=Name(id='f', ctx="
                 "Store(), annotation=None, type_comment=None), body=[Pass()])]"
                 ", orelse=[Pass()], finalbody=[])], type_ignores=[])")
-        self.assertEqual(gast.dump(tree), norm)
+        self.assertEqual(dump(tree), norm)
 
     def test_Raise(self):
         codes = ('raise Exception',
@@ -184,10 +329,9 @@ class CompatTestCase(unittest.TestCase):
         for code, norm in zip(codes, norms):
             tree = gast.parse(code)
             compile(gast.gast_to_ast(tree), '<test>', 'exec')
-            self.assertEqual(gast.dump(tree), norm)
+            self.assertEqual(dump(tree), norm)
 
     def test_Call(self):
-        self.maxDiff = None
         code = 'foo(x, y=1, *args, **kwargs)'
         tree = gast.parse(code)
         compile(gast.gast_to_ast(tree), '<test>', 'exec')
@@ -200,7 +344,7 @@ class CompatTestCase(unittest.TestCase):
                 "arg='y', value=Constant(value=1, kind=None)), keyword(arg"
                 "=None, value=Name(id='kwargs', ctx=Load(), annotation=None, "
                 "type_comment=None))]))], type_ignores=[])")
-        self.assertEqual(gast.dump(tree), norm)
+        self.assertEqual(dump(tree), norm)
 
     def test_With(self):
         code = 'with open("any"): pass'
@@ -211,7 +355,7 @@ class CompatTestCase(unittest.TestCase):
                 "type_comment=None), args=[Constant(value='any', "
                 "kind=None)], keywords=[]), optional_vars=None)], body=["
                 "Pass()], type_comment=None)], type_ignores=[])")
-        self.assertEqual(gast.dump(tree), norm)
+        self.assertEqual(dump(tree), norm)
 
     def test_TryFinally(self):
         code = 'try:pass\nfinally:pass'
@@ -219,7 +363,7 @@ class CompatTestCase(unittest.TestCase):
         compile(gast.gast_to_ast(tree), '<test>', 'exec')
         norm = ("Module(body=[Try(body=[Pass()], handlers=[], orelse=[], "
                 "finalbody=[Pass()])], type_ignores=[])")
-        self.assertEqual(gast.dump(tree), norm)
+        self.assertEqual(dump(tree), norm)
 
     def test_star_argument(self):
         code = 'def foo(*a): pass'
@@ -230,8 +374,8 @@ class CompatTestCase(unittest.TestCase):
                 "annotation=None, type_comment=None), "
                 "kwonlyargs=[], kw_defaults=[], kwarg=None, defaults=[]), "
                 "body=[Pass()], decorator_list=[], returns=None, "
-                "type_comment=None)], type_ignores=[])")
-        self.assertEqual(gast.dump(tree), norm)
+                "type_comment=None, type_params=[])], type_ignores=[])")
+        self.assertEqual(dump(tree), norm)
 
     def test_keyword_argument(self):
         code = 'def foo(**a): pass'
@@ -241,9 +385,9 @@ class CompatTestCase(unittest.TestCase):
                 "posonlyargs=[], vararg=None, kwonlyargs=[], kw_defaults=[], "
                 "kwarg=Name(id='a', ctx=Param(), annotation=None, "
                 "type_comment=None), defaults=[]), body=[Pass()], "
-                "decorator_list=[], returns=None, type_comment=None)], "
-                "type_ignores=[])")
-        self.assertEqual(gast.dump(tree), norm)
+                "decorator_list=[], returns=None, type_comment=None, "
+                "type_params=[])], type_ignores=[])")
+        self.assertEqual(dump(tree), norm)
 
     def test_Index(self):
         code = 'def foo(a): a[1]'
@@ -255,12 +399,11 @@ class CompatTestCase(unittest.TestCase):
                 ", kwarg=None, defaults=[]), body=[Expr(value=Subscript(value="
                 "Name(id='a', ctx=Load(), annotation=None, type_comment=None)"
                 ", slice=Constant(value=1, kind=None), ctx=Load()"
-                "))], decorator_list=[], returns=None, type_comment=None)]"
-                ", type_ignores=[])")
-        self.assertEqual(gast.dump(tree), norm)
+                "))], decorator_list=[], returns=None, type_comment=None, "
+                "type_params=[])], type_ignores=[])")
+        self.assertEqual(dump(tree), norm)
 
     def test_ExtSlice(self):
-        self.maxDiff = None
         code = 'def foo(a): a[:,:]'
         tree = gast.parse(code)
         compile(gast.gast_to_ast(tree), '<test>', 'exec')
@@ -272,11 +415,10 @@ class CompatTestCase(unittest.TestCase):
                 ", slice=Tuple(elts=[Slice(lower=None, upper=None, step="
                 "None), Slice(lower=None, upper=None, step=None)], ctx=Load())"
                 ", ctx=Load()))], decorator_list=[], returns=None, "
-                "type_comment=None)], type_ignores=[])")
-        self.assertEqual(gast.dump(tree), norm)
+                "type_comment=None, type_params=[])], type_ignores=[])")
+        self.assertEqual(dump(tree), norm)
 
     def test_ExtSlices(self):
-        self.maxDiff = None
         code = 'def foo(a): a[1,:]'
         tree = gast.parse(code)
         compile(gast.gast_to_ast(tree), '<test>', 'exec')
@@ -288,11 +430,10 @@ class CompatTestCase(unittest.TestCase):
                 ", slice=Tuple(elts=[Constant(value=1, kind="
                 "None), Slice(lower=None, upper=None, step=None)], ctx=Load())"
                 ", ctx=Load()))], decorator_list=[], returns=None, "
-                "type_comment=None)], type_ignores=[])")
-        self.assertEqual(gast.dump(tree), norm)
+                "type_comment=None, type_params=[])], type_ignores=[])")
+        self.assertEqual(dump(tree), norm)
 
     def test_Ellipsis(self):
-        self.maxDiff = None
         code = 'def foo(a): a[...]'
         tree = gast.parse(code)
         compile(gast.gast_to_ast(tree), '<test>', 'exec')
@@ -302,12 +443,11 @@ class CompatTestCase(unittest.TestCase):
                 ", kwarg=None, defaults=[]), body=[Expr(value=Subscript(value="
                 "Name(id='a', ctx=Load(), annotation=None, type_comment=None)"
                 ", slice=Constant(value=Ellipsis, kind=None), ctx=Load()))], "
-                "decorator_list=[], returns=None, type_comment="
-                "None)], type_ignores=[])")
-        self.assertEqual(gast.dump(tree), norm)
+                "decorator_list=[], returns=None, type_comment=None, "
+                "type_params=[])], type_ignores=[])")
+        self.assertEqual(dump(tree), norm)
 
     def test_ExtSliceEllipsis(self):
-        self.maxDiff = None
         code = 'def foo(a): a[1, ...]'
         tree = gast.parse(code)
         compile(gast.gast_to_ast(tree), '<test>', 'exec')
@@ -319,8 +459,8 @@ class CompatTestCase(unittest.TestCase):
                 ", slice=Tuple(elts=[Constant(value=1, kind=None)"
                 ", Constant(value=Ellipsis, kind=None)], ctx=Load()), ctx="
                 "Load()))], decorator_list=[], returns=None, type_comment="
-                "None)], type_ignores=[])")
-        self.assertEqual(gast.dump(tree), norm)
+                "None, type_params=[])], type_ignores=[])")
+        self.assertEqual(dump(tree), norm)
 
 
 if __name__ == '__main__':
